#!/usr/bin/env perl
# -*- perl -*-

#
# $Id: BBBikeRouting.pm,v 1.9 2002/07/11 19:43:33 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 2000,2001 Slaven Rezic. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: slaven.rezic@berlin.de
# WWW:  http://www.rezic.de/eserte/
#

package BBBikeRouting;

BEGIN { $^W = 1 }

use strict;
use Class::Struct;
use BBBikeUtil;

struct('BBBikeRouting::Position' => {Street => "\$", Citypart => "\$",
				     Coord => "\$", Multi => "\$",
				    });
struct('BBBikeRouting::Context' => {Vehicle => "\$", Scope => "\$",
				    Velocity => "\$",
				    UseXS => "\$", UseCache => "\$",
				    UseNetServer => "\$",
				    ZIPLookArgs => "\$",
				    SearchArgs => "\$", Algorithm => "\$",
				    CGI => "\$", BrowserInfo => "\$",
				    RouteInfoKm => "\$",
				    Verbose => "\$",
				   });
struct('BBBikeRouting' => {Context => "BBBikeRouting::Context",
			   Start => "BBBikeRouting::Position",
			   Via => "\$", # array of BBBikeRouting::Position
			   Goal => "BBBikeRouting::Position",
			   Streets => "\$", ZIP => "\$",
			   ZIPStreets => "\$", Net => "\$",
			   Stations => "\$",
			   Crossings => "\$",
			   Path => "\$", RouteInfo => "\$",
			   #PenaltyNets => "\$",
			  });

sub BBBikeRouting::LastVia {
    my $self = shift;
    if (ref $self->Via eq 'ARRAY') {
	$self->Via->[-1];
    } else {
	undef;
    }
}

sub init_context {
    my $self = shift;
    my $context = $self->Context;
    $context->Vehicle("bike");
    $context->Velocity(kmh2ms(20));
    $context->Scope("city");
    $context->UseXS(1);
    $context->UseNetServer(1);
    $context->UseCache(1);
    $context->Algorithm("C-A*");
    $context->RouteInfoKm(1);
    $self;
}

sub init_str {
    my $self = shift;
    if (!$self->Streets) {
	my $context = $self->Context;
	require Strassen::Core;
	if ($context->Vehicle eq 'oepnv') {
	    my $sstr = Strassen->new("sbahn");
	    $sstr = Strassen->new_copy_restricted($sstr,
						  -restrictions => [qw/S0/]);
	    my $ustr = Strassen->new("ubahn");
	    $ustr = Strassen->new_copy_restricted($ustr,
						  -restrictions => [qw/U0/]);
	    require Strassen::MultiStrassen;
	    $self->Streets(MultiStrassen->new($sstr, $ustr));
	} else {
	    if ($context->Scope eq 'region') {
		require Strassen::MultiStrassen;
		$self->Streets(MultiStrassen->new
			       (Strassen->new("strassen"),
				Strassen->new("landstrassen"),
			       ));
	    } elsif ($context->Scope eq 'wideregion') {
		require Strassen::MultiStrassen;
		$self->Streets(MultiStrassen->new
			       (Strassen->new("strassen"),
				Strassen->new("landstrassen"),
				Strassen->new("landstrassen2"),
			       ));
	    } else {
		$self->Streets(Strassen->new("strassen"));
	    }
	    if ($context->Vehicle eq 'car') {
		$self->Streets(Strassen->new_copy_restricted
			       ($self->Streets, -restrictions => [qw/NN/]));
	    }
	}
    }
    $self->Streets;
}

sub init_zip {
    my $self = shift;
    if (!$self->ZIP) {
	require PLZ;
	$self->ZIP(PLZ->new());
    }
    $self->ZIP;
}

sub init_zip_s {
    my $self = shift;
    if (!$self->ZIPStreets) {
	$self->ZIPStreets($self->init_zip->as_streets);
    }
    $self->ZIPStreets;
}

sub init_net {
    my $self = shift;
    if (!$self->Net) {
	require Strassen::StrassenNetz;
	my $context = $self->Context;
	$self->init_str;
	if ($context->UseXS) {
	    eval q{ use BBBikeXS };
	}
	if ($context->Vehicle eq 'oepnv') {
	    $self->Net(StrassenNetz->new($self->Streets));
	    die "NYI XXX" if $context->Algorithm eq 'C-A*-2';
	    $self->Net->make_net(UseCache => $context->UseCache);
	    $self->init_stations;
	    $self->Net->add_umsteigebahnhoefe($self->Stations,
					      -addmapfile => 'umsteigebhf');
	} else {
	    $self->Net(StrassenNetz->new_from_best
		       (Strassen => $self->Streets,
			OnCreate => sub {
			    if ($context->Algorithm eq 'C-A*-2') {
				require Strassen::CNetFileDist;
				Strassen::CNetFile::make_net($_[0]);
				    $_[0]->make_sperre
					('gesperrt',
					 Type => ['einbahn', 'sperre',
						  'wegfuehrung']);
				# XXX make_sperre nyi
			    } else {
				$_[0]->make_net(UseCache => $context->UseCache);
				if ($context->Vehicle eq 'bike') {
				    $_[0]->make_sperre
					('gesperrt',
					 Type => ['einbahn', 'sperre',
						  'wegfuehrung']);
				} elsif ($context->Vehicle eq 'car') {
				    $_[0]->make_sperre
					('gesperrt',
					 Type => ['einbahn', 'sperre',
						  'tragen', 'wegfuehrung']);
				    $_[0]->make_sperre
					('gesperrt_car',
					 Type => ['einbahn', 'sperre',
						  'tragen', 'wegfuehrung']);
				}
			    }
			},
			NoNewFromServer => !$context->UseNetServer,
		       ));
	}
    }
    $self->Net;
}

sub init_crossings {
    my $self = shift;
    if (!$self->Crossings) {
	require Strassen::Kreuzungen;
	my $context = $self->Context;
	if ($context->Vehicle eq 'oepnv') {
	    $self->init_stations;
	    $self->Crossings(Kreuzungen->new_from_strassen(Strassen => $self->Stations, WantPos => 1, UseCache => $context->UseCache));
	} else {
	    $self->init_str;
	    $self->Crossings(Kreuzungen->new(Strassen => $self->Streets, WantPos => 1, UseCache => $context->UseCache));
	}
	$self->Crossings->make_grid;
    }
    $self->Crossings;
}

sub init_stations {
    my $self = shift;
    if (!$self->Stations) {
	require Strassen::Core;
	require Strassen::MultiStrassen;
	my $sbhf = Strassen->new("sbahnhof");
	my $ubhf = Strassen->new("ubahnhof");
	$self->Stations(MultiStrassen->new($sbhf, $ubhf));
    }
    $self->Stations;
}

foreach (qw(Start Goal)) {
    my $c='sub get_'.lc($_).'_position { shift->get_position(\''.$_.'\', @_) }';
#    warn $c;
    eval $c;
}

sub get_position {
    my $self = shift;
    my $type = ucfirst(shift); # start or goal
    my $pos_o = $self->$type();
    my $street = shift || $pos_o->Street;
    my $citypart = shift || $pos_o->Citypart;

    $self->init_zip;
    $self->init_net;

    my $context = $self->Context;

    my $return_multiple = 0; # XXX
    my(@from_res) = $self->ZIP->look_loop_best
	(PLZ::split_street($street),
	 MultiZIP => !$return_multiple,
	 MultiCitypart => !$return_multiple,
	 Agrep => 'default',
	 (defined $citypart ? (Citypart => $citypart) : ()),
	 ($context->ZIPLookArgs ? @{ $context->ZIPLookArgs } : ()),
	);
    return undef if (!@{ $from_res[0] });

#XXX
#      $multi_from = @{ $from_res[0] } > 1;
#      if ($return_multiple && $multi_from) {
#  	return $from_res[0];
#      }

    my $from_data = $from_res[0]->[0];
    $pos_o->Street  ($from_data->[PLZ::LOOK_NAME()]);
    $pos_o->Citypart($from_data->[PLZ::LOOK_CITYPART()]);
    $pos_o->Coord   ($from_data->[PLZ::LOOK_COORD()]);

    $self->fix_position($type);
}

sub fix_position {
    my($self, $type) = @_;
    $self->init_net;
    if (!$self->Net->reachable($self->$type()->Coord)) {
	$self->init_crossings;
	$self->$type()->Coord(($self->Crossings->nearest_loop(split /,/, $self->$type()->Coord))[0]);
    }
    $self->$type()->Coord;
}

sub search {
    my $self = shift;

    $self->init_zip;
    $self->init_net;

    my $continued = 0;
    my $start_coord;
    if (ref $self->Via eq 'ARRAY' && @{$self->Via} > 0) {
	$self->get_position("LastVia") if $self->LastVia && !$self->LastVia->Coord;
	$start_coord = $self->LastVia->Coord;
	$continued = 1;
    } else {
	$self->get_position("Start") if !$self->Start->Coord;
	$start_coord = $self->Start->Coord;
    }
    $self->get_position("Goal") if !$self->Goal->Coord;

    die "No start coordinate after using get_position" if !$start_coord;
    die "No goal coordinate after using get_position" if !$self->Goal->Coord;

    my $context = $self->Context;

    my($res) = $self->Net->search
	($start_coord, $self->Goal->Coord,
	 Tragen => ($context->Vehicle eq 'bike'),
	 $context->Velocity ? (Velocity => $context->Velocity) : (),
	 $context->SearchArgs ? @{ $context->SearchArgs } : (),
	 $context->Algorithm ? (Algorithm => $context->Algorithm) : (),
	 $context->Verbose ? (Stat => 1) : (),
	);

    if ($continued && $self->Path) {
	if (defined $res) {
	    $self->Path([@{ $self->Path },
			 @{ $res }]);
	}
	my $whole = $self->RouteInfo->[-1]->{Whole};
	my @new_route_info = $self->Net->route_info(Route => $res,
						    Km    => $context->RouteInfoKm);
	$_->{Whole} = $_->{Whole}+$whole for @new_route_info;
	$self->RouteInfo([@{ $self->RouteInfo }, @new_route_info ]);
    } else {
	$self->Path([]);
	if (defined $res) {
	    $self->Path($res);
	}
	$self->RouteInfo([$self->Net->route_info(Route => $self->Path,
						 Km    => $context->RouteInfoKm)]);
    }
}

sub continue {
    my($self, $position) = @_;
    $self->Via([]) if ref $self->Via ne 'ARRAY';
    push @{ $self->Via }, $self->Goal;
    $self->Goal($position);
}

sub inc {
    eval <<'EOF';
use FindBin;
use lib ("$FindBin::RealBin",
	 "$FindBin::RealBin/lib",
	 "$FindBin::RealBin/data",
	 "$FindBin::RealBin/..",
	 "$FindBin::RealBin/../lib",
	 "$FindBin::RealBin/../data",
	);
EOF
    warn $@ if $@;
}

1;

__END__

