# -*- perl -*-

#
# $Id: DBase.pm,v 1.9 2002/01/24 07:38:29 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 2001 Slaven Rezic. All rights reserved.
#
# Mail: slaven.rezic@berlin.de
# WWW:  http://www.rezic.de/~eserte/
#

package ESRI::Shapefile::DBase;

use strict;
use vars qw(@ISA);

use Class::Accessor; # workaround 5.00503 bug (???)
use base qw(Class::Accessor::Fast);

__PACKAGE__->mk_accessors(qw(File Root Data DBH XBase Init Fields FieldsHash));

sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = {};
    bless $self, $class;

    my(%args) = @_;

    if ($args{-file}) {
	$self->set_file($args{-file});
    }
    if ($args{-root}) {
	$self->Root($args{-root});
    }
    $self->Init(0);

    $self;
}

sub set_file {
    my($self, $file) = @_;

    $self->File($file);

    open(F, $file) or die "Can't open $file: $!";

    close F;
}

sub init {
    my $self = shift;
    if (!$self->Init) {
	my @data;
	my $sth = $self->_get_all_sth;
	while(my @row = $sth->fetchrow_array) {
	    cp850_iso($_) foreach (@row);
	    push @data, \@row;
	}
	$self->Data(\@data);
	$self->_finish_db;
	$self->Init(1);
    }
    $self;
}

sub get_fields {
    my $self = shift;

    require XBase;
    if (!$self->XBase) {
	$self->XBase(XBase->new($self->File));
	die "Can't create XBase object from @{[ $self->File ]}"
	    if !$self->XBase;
    }

    if (!$self->Fields) {
	$self->Fields([ $self->XBase->field_names ]);
    }

    if (!$self->FieldsHash) {
	my %h;
	for my $i (0 .. scalar @{ $self->Fields }) {
	    $h{$self->Fields->[$i]} = $i;
	}
	$self->FieldsHash(\%h);
    }

    @{ $self->Fields };
}

sub _get_all_sth {
    my $self = shift;

    require File::Basename;
    require DBI;

    my($name, $dir) = File::Basename::fileparse($self->File, '\..*');
    my $dbh = DBI->connect("DBI:XBase:$dir") or die $DBI::errstr;
    my $sth = $dbh->prepare("select * from $name") or die $dbh->errstr;
    $sth->execute or die $sth->errstr;

    $self->DBH($dbh);

    $sth;
}

sub _finish_db {
    my $self = shift;
    if ($self->DBH) {
	$self->DBH->disconnect;
	$self->DBH(undef);
    }
}

sub cp850_iso {
    $_[0] =~ tr/\204\224\201\216\231\232\341\202\370//;
}

return 1 if caller();

######################################################################

if ($0 =~ /merge_with_bbd/) {
    my $shapefile = shift or die "Shapefile?";
    my $bbdfile   = shift or die "bbd file?";
    my $outfile   = shift or die "new bbd file for output?";

    require ESRI::Shapefile;
    my $esri = ESRI::Shapefile->new;
    $esri->set_file($shapefile);
    $esri->DBase->merge_with_bbd($bbdfile, $outfile);
}

__END__
