# -*- perl -*-

#
# $Id: Inline.pm,v 2.7 2002/07/13 21:02:59 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 2001 Slaven Rezic. All rights reserved.
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.
#
# Mail: slaven.rezic@berlin.de
# WWW:  http://www.rezic.de/eserte/
#

# ***FILTER=1***
package Strassen::Inline;
# ***FILTER=2***
#package Strassen::Inline2;
# ***FILTER=all***
require 5.005; # new semantics of hv_iterinit

BEGIN {
    $VERSION = sprintf("%d.%02d", q$Revision: 2.7 $ =~ /(\d+)\.(\d+)/);
}

use Inline 0.40; # because of API changes
use Config;
use Inline C => Config => CCFLAGS => "-O2";
#use Inline Config => CLEAN_AFTER_BUILD => 0; #XXX debugging
#use Inline C => Config => CCFLAGS => "-g -O2"; #XXX debugging
use Inline (C => DATA =>
# ***FILTER=1***
	    NAME => 'Strassen::Inline',
# ***FILTER=2***
#	    NAME => 'Strassen::Inline2',
#	    CCFLAGS => "-DUSE_MMAP_IMPL $Config{'ccflags'}",
# ***FILTER=all***
	    VERSION => $VERSION,
	   );

1;

__DATA__
__C__

typedef int dist_t;

typedef struct {
#ifdef USE_MMAP_IMPL
  int predecessor;
#else
  char* predecessor;
#endif
  dist_t dist;
  dist_t heuristic_dist;
} search_node;

#define SQR(a) ((a)*(a))

#ifdef USE_MMAP_IMPL
inline dist_t _strecke(int x1, int y1, int x2, int y2) {
  return (dist_t)(sqrt(SQR(x2-x1) + SQR(y2-y1)));
}
#else
inline dist_t _strecke(char* p1, char* p2) {
  int x1 = atoi(p1);
  int y1 = atoi(strchr(p1, ',')+1);
  int x2 = atoi(p2);
  int y2 = atoi(strchr(p2, ',')+1);
  return (dist_t)(sqrt(SQR(x2-x1) + SQR(y2-y1)));
}
#endif

#ifdef USE_MMAP_IMPL
void _dump_ptr(int c_net_mmap, int ptr) {
  int* mmap_ptr = (int*)(c_net_mmap+ptr);
  int x = *(mmap_ptr++);
  int y = *(mmap_ptr++);
  /*
  DBT key, val;
  search_node *sn;
  key.data = &ptr;
  key.size = sizeof(int);
  NODES->get(NODES, &key, &val, 0);
  sn = (search_node*)val.data;*/
  fprintf(stderr, "x/y=%d/%d dist=%d hdist=%d\n", x, y, 0, 0 /*sn->dist, sn->heuristic_dist*/);
}
#else
void _dump_ptr(int d1, int d2) { }
#endif

#ifdef USE_MMAP_IMPL
# define COORD_HV_VAL(x) ((char*)(&x))
# define COORD_HV_LEN(x) (sizeof(int))
# define DECLARE_HV_VAL(x) \
	int x = 0; \
	int x##_len = 0
#else
# define COORD_HV_VAL(x) (x)
# define COORD_HV_LEN(x) (strlen(x))
# define DECLARE_HV_VAL(x) \
	char* x = NULL; \
	int x##_len = 0
#endif

#define SEARCH_C_ARGS 3
void search_c(SV* self, char* from, char* to, ...) {
  HV* self_hash = NULL;

#ifdef USE_MMAP_IMPL
  int c_net_mmap = 0;
  HV* c_net_coord2ptr = NULL;
#else
  HV* net = NULL;
#endif
  HV* wegfuehrung = NULL;

  HV* OPEN   = newHV();
  HV* CLOSED = newHV();
  HV* NODES  = newHV();

#ifdef USE_MMAP_IMPL
  int from_ptr, to_ptr;
  int from_x, from_y;
  int to_x, to_y;
#endif

  SV* tmp;
  SV* penaltysub = NULL;

  int i;
  Inline_Stack_Vars;

  if ((Inline_Stack_Items-SEARCH_C_ARGS)%2 != 0)
    croak("usage: $net->search_c($from, $to, -opt => $val, ...) [%d items]", Inline_Stack_Items);
  for (i = SEARCH_C_ARGS; i < Inline_Stack_Items; i+=2) {
    char* opt = SvPV(Inline_Stack_Item(i), PL_na);
    if (strcmp(opt, "-penaltysub") == 0) {
      penaltysub = newSVsv(Inline_Stack_Item(i+1)); // XXX cleanup? newsvsv needed???
    } else {
      croak("Invalid option %s", opt);
    }
  }

  if (!sv_isobject(self) || !sv_derived_from(self, "StrassenNetz"))
    croak("Invalid object supplied to search_c");

  self_hash = (HV*)SvRV(self);

#ifdef USE_MMAP_IMPL
  {
    SV** tmp = hv_fetch(self_hash, "CNetMmap", strlen("CNetMmap"), 0);
    if (tmp)
      c_net_mmap = SvIV(*tmp);
    else
      croak("No CNetMmap element (or invalid type) in object hash");
  }

  {
    SV** tmp = hv_fetch(self_hash, "CNetCoord2Ptr", strlen("CNetCoord2Ptr"), 0);
    if (tmp && SvTYPE(SvRV(*tmp)) == SVt_PVHV)
      c_net_coord2ptr = (HV*)SvRV(*tmp);
    else
      croak("No CNetCoord2Ptr element (or invalid type) in object hash");
  }
#else
  {
    SV** tmp = hv_fetch(self_hash, "Net", strlen("Net"), 0);
    if (tmp && SvTYPE(SvRV(*tmp)) == SVt_PVHV)
      net = (HV*)SvRV(*tmp);
    else
      croak("No Net element (or invalid type) in object hash");
  }
#endif

  {
    SV** tmp = hv_fetch(self_hash, "Wegfuehrung", strlen("Wegfuehrung"), 0);
    if (tmp && SvTYPE(SvRV(*tmp)) == SVt_PVHV)
      wegfuehrung = (HV*)SvRV(*tmp);
  }

#ifdef USE_MMAP_IMPL
  {
    SV** tmp = hv_fetch(c_net_coord2ptr, from, strlen(from), 0);
    if (!tmp)
      croak("Cannot find start coordinate %s", from);
    from_ptr = SvIV(*tmp);

    from_x = *((int*)(c_net_mmap+from_ptr));
    from_y = *(((int*)(c_net_mmap+from_ptr))+1);

    tmp = hv_fetch(c_net_coord2ptr, to, strlen(to), 0);
    if (!tmp)
      croak("Cannot find goal coordinate %s", to);
    to_ptr = SvIV(*tmp);

    to_x = *((int*)(c_net_mmap+to_ptr));
    to_y = *(((int*)(c_net_mmap+to_ptr))+1);
  }
#endif

  {
    search_node *first_node = malloc(sizeof(search_node));
    if (!first_node)
      croak("Can't malloc");
    first_node->predecessor = NULL;
#ifdef USE_MMAP_IMPL
    first_node->heuristic_dist = _strecke(from_x, from_y, to_x, to_y);
#else
    first_node->heuristic_dist = _strecke(from, to);
#endif
    first_node->dist = 0;

#ifdef USE_MMAP_IMPL
    hv_store(OPEN,  COORD_HV_VAL(from_ptr), COORD_HV_LEN(from_ptr), newSViv(1), 0);
    hv_store(NODES, COORD_HV_VAL(from_ptr), COORD_HV_LEN(from_ptr), newSViv((IV)first_node), 0);
#else
    hv_store(OPEN,  COORD_HV_VAL(from), COORD_HV_LEN(from), newSViv(1), 0);
    hv_store(NODES, COORD_HV_VAL(from), COORD_HV_LEN(from), newSViv((IV)first_node), 0);
#endif
  }

  while(1) {
    if (hv_iterinit(OPEN) == 0) {
      Inline_Stack_Reset;
      Inline_Stack_Done;
    }

    {
      DECLARE_HV_VAL(min_node);
      double min_node_f;
      HE* OPEN_he;
      while(OPEN_he = hv_iternext(OPEN)) {
	int got_better = 0;
	DECLARE_HV_VAL(key);
	search_node* sn;
	SV** node_sv;

#ifdef USE_MMAP_IMPL
	key = *((int*)(HePV(OPEN_he, key_len)));
	node_sv = hv_fetch(NODES, COORD_HV_VAL(key), COORD_HV_LEN(key), 0);
#else
	key = HePV(OPEN_he, key_len);
	node_sv = hv_fetch(NODES, COORD_HV_VAL(key), key_len, 0);
#endif

	if (!node_sv)
#ifdef USE_MMAP_IMPL
	  croak("Can't get node for key <%d>", key);
#else
	  croak("Can't get node for key <%s>", key);
#endif
	sn = (search_node*)SvIV(*node_sv);
	if (min_node == NULL)
	  got_better = 1;
	else if (sn->heuristic_dist < min_node_f)
	  got_better = 1;

	if (got_better) {
	  min_node = key;
	  min_node_len = key_len;
	  min_node_f = sn->heuristic_dist;
	}
      }

      if (min_node == 0)
	  croak("No minimal node found");

      /* move min_node from OPEN to CLOSED */
      hv_store(CLOSED, COORD_HV_VAL(min_node), min_node_len, newSViv(1), 0);
      hv_delete(OPEN, COORD_HV_VAL(min_node), min_node_len, G_DISCARD);

#ifdef USE_MMAP_IMPL
      if (min_node == to_ptr) {
#else
      if (strcmp(min_node, to) == 0) {
#endif

	AV* path = newAV();
	dist_t len = 0;
	SV* verbose = perl_get_sv("StrassenNetz::VERBOSE", FALSE);
	if (verbose && SvTRUE(verbose))
	  fprintf(stderr, "Path found.\n");
	while(1) {
	  SV** tmp;
	  search_node *sn;
	  AV* xy = newAV();
	  int x,y;
#ifdef USE_MMAP_IMPL
	  int *mmap_ptr = (int*)(c_net_mmap + min_node);
	  x = *(mmap_ptr++);
	  y = *mmap_ptr;
#else
	  x = atoi(min_node);
	  y = atoi(strchr(min_node,',')+1);
#endif
	  av_push(xy, newSViv(x));
	  av_push(xy, newSViv(y));

	  //  fprintf(stderr,"<%s>\n", min_node);
	  av_unshift(path, 1);
	  av_store(path, 0, newRV_noinc((SV*)xy));
	  tmp = hv_fetch(NODES, COORD_HV_VAL(min_node), COORD_HV_LEN(min_node), 0);
	  if (!tmp) break;
	  sn = (search_node*)SvIV(*tmp);
	  if (sn->predecessor == NULL) break;
#ifdef USE_MMAP_IMPL
	  {
	    int x2,y2;
	    int *mmap_ptr = (int*)(c_net_mmap + sn->predecessor);
	    x2 = *(mmap_ptr++);
	    y2 = *mmap_ptr;
	    len += _strecke(x,y,x2,y2);
	  }
#else
	  len += _strecke(min_node, sn->predecessor);
#endif
	  min_node = sn->predecessor;
	}

	//XXX $visited_nodes = scalar(keys %OPEN) + scalar(keys %CLOSED);

	// XXX free everything in NODES

	// XXX use sv_2mortal here?
	Inline_Stack_Reset;
	Inline_Stack_Push(newRV_noinc((SV*)path));
	Inline_Stack_Push(newSViv((int)len));
	Inline_Stack_Push(newSViv(0));
	Inline_Stack_Push(newSViv(0));
	Inline_Stack_Push(&PL_sv_undef);
	Inline_Stack_Done;

	return;
      }

      {
#ifdef USE_MMAP_IMPL
	int *mmap_ptr = (int*)(c_net_mmap + min_node);
	int no_succ;
	int succ_i;
	SV** node_sv = hv_fetch(NODES, COORD_HV_VAL(min_node), min_node_len, 0);
	search_node* sn = (search_node*)SvIV(*node_sv);
	//XXXfprintf(stderr, "<%d><%d><%d>x=<%d> y=<%d> no=<%d> hdist=<%d>\n", (int)c_net_mmap, min_node,mmap_ptr, *mmap_ptr,*(mmap_ptr+1),*(mmap_ptr+2), sn->heuristic_dist );
	mmap_ptr+=2; /* skip x, y coordinate */
	no_succ = *(mmap_ptr++);
	for(succ_i=0; succ_i<no_succ; succ_i++) {
	  int succ_key = *(mmap_ptr++);
	  int succ_key_len = sizeof(int);
	  dist_t len_pen = *(mmap_ptr++);
#else
	HE* succ_he;
	SV** tmp = hv_fetch(net, min_node, min_node_len, 0);
	HV* min_node_net = (HV*)SvRV(*tmp);
	SV** node_sv = hv_fetch(NODES, min_node, min_node_len, 0);
	search_node* sn = (search_node*)SvIV(*node_sv);
	while(succ_he = hv_iternext(min_node_net)) {
	  int succ_key_len;
	  char* succ_key = HePV(succ_he, succ_key_len);
	  dist_t len_pen;
#endif

	  /* do not check against the predecessor of this node */
#ifdef USE_MMAP_IMPL
	  if (sn->predecessor != 0 && sn->predecessor == succ_key)
	    continue;
#else
	  if (sn->predecessor != NULL && strcmp(sn->predecessor, succ_key) == 0)
	    continue;
#endif

	  if (wegfuehrung) {
	    SV** tmp = hv_fetch(wegfuehrung, COORD_HV_VAL(succ_key), succ_key_len, 0);
	    if (tmp) {
#ifdef USE_MMAP_IMPL
	      char this_node_char[20]; // XXX should be enough to hold x,y
#endif
	      int do_continue = 0;
	      AV* wegfuehrungen = (AV*)SvRV(*tmp);
	      int j;
	      for(j=0; j<=av_len(wegfuehrungen); j++) {
		SV** tmp = av_fetch(wegfuehrungen, j, 0);
		AV* wegf = (AV*)SvRV(*tmp);
#ifdef USE_MMAP_IMPL
		int this_node = min_node;
#else
		char* this_node = min_node;
#endif
		int same = 1;
		int i;
		for(i=av_len(wegf)-1; i>=0; i--) {
		  tmp = av_fetch(wegf, i, 0);
#ifdef USE_MMAP_IMPL
		  sprintf(this_node_char, "%d,%d",
			  *((int*)(c_net_mmap + min_node)),
			  *(((int*)(c_net_mmap + min_node))+1));
		  if (strcmp(SvPV(*tmp, PL_na), this_node_char) != 0) {
#else
		  if (strcmp(SvPV(*tmp, PL_na), this_node) != 0) {
#endif
		    same = 0;
		    break;
		  }
		  {
		    search_node *this_sn;
		    tmp = hv_fetch(NODES, COORD_HV_VAL(this_node), COORD_HV_LEN(this_node), 0);
		    this_sn = (search_node*)SvIV(*tmp);
		    if (this_sn->predecessor == NULL) {
		      same = 0;
		      break;
		    }
		    this_node = this_sn->predecessor;
		  }
		}
		if (same) {
		  do_continue = 1;
		  break;
		}
	      }
	      if (do_continue) continue;
	    }
	  }

	  /* XXX Statistik etc. */

	  {
	    dist_t g, f;
#ifndef USE_MMAP_IMPL
	    len_pen = SvIV(HeVAL(succ_he));
#endif

	    if (penaltysub != NULL) {
	      // call penaltysub with
	      // * min_node
	      // * last_node
	      // * len_pen
	      // returned value is new len_pen

	      SV* last_node = &PL_sv_undef;

	      /* Using Inline_Stack_Reset et al. as advertised in
		 Inline::C-Cookbook does not work! */
	      ENTER;
	      SAVETMPS;

	      if (sn->predecessor != NULL) {
#ifdef USE_MMAP_IMPL
		last_node = newSVpv("", 0);
		sv_catpvf(last_node, "%d,%d", 
			  *((int*)(c_net_mmap + sn->predecessor)),
			  *(((int*)(c_net_mmap + sn->predecessor))+1));
		sv_2mortal(last_node);
#else
		last_node = sv_2mortal(newSVpv(sn->predecessor, 0));
#endif
	      }

	      PUSHMARK(SP);
#ifdef USE_MMAP_IMPL
	      {
		SV* min_node_sv = newSVpv("", 0);
		sv_catpvf(min_node_sv, "%d,%d", 
			  *((int*)(c_net_mmap + min_node)),
			  *(((int*)(c_net_mmap + min_node))+1));
		Inline_Stack_Push(sv_2mortal(min_node_sv));
	      }
#else
	      Inline_Stack_Push(sv_2mortal(newSVpv(min_node, min_node_len)));
#endif
	      Inline_Stack_Push(last_node);
	      Inline_Stack_Push(sv_2mortal(newSViv(len_pen))); // XXX depends on dist_t!!!
	      PUTBACK;
	      if (perl_call_sv(penaltysub, G_SCALAR) != 1)
		croak("perl_call_sv return value should be 1");

	      SPAGAIN;
	      len_pen = POPi;

	      PUTBACK;
	      FREETMPS;
	      LEAVE;
	      
	    }

	    g = sn->dist + len_pen;
#ifdef USE_MMAP_IMPL
	    {
	      int succ_x = *((int*)(c_net_mmap+succ_key));
	      int succ_y = *(((int*)(c_net_mmap+succ_key))+1);
	      f = g + _strecke(succ_x, succ_y, to_x, to_y);
	    }
#else
	    f = g + _strecke(succ_key, to);
#endif

	    // !exists in OPEN and !exists in CLOSED
	    if (!hv_exists(NODES, COORD_HV_VAL(succ_key), succ_key_len)) {
	      search_node* new_sn = malloc(sizeof(search_node));
	      if (!new_sn) croak("Cannot malloc");
	      new_sn->predecessor = min_node;
	      new_sn->dist = g;
	      new_sn->heuristic_dist = f;
	      hv_store(NODES, COORD_HV_VAL(succ_key), succ_key_len, newSViv((IV)new_sn), 0);
	      hv_store(OPEN, COORD_HV_VAL(succ_key), succ_key_len, newSViv(1), 0);
	    } else {
	      SV** tmp = hv_fetch(NODES, COORD_HV_VAL(succ_key), succ_key_len, 0);
	      search_node* old_sn = (search_node*)SvIV(*tmp);
	      if (f < old_sn->heuristic_dist) {
		old_sn->predecessor = min_node; /*XXX free old predecessor? */
		old_sn->dist = g;
		old_sn->heuristic_dist = f;
		if (hv_exists(CLOSED, COORD_HV_VAL(succ_key), succ_key_len)) {
		  hv_store(OPEN, COORD_HV_VAL(succ_key), succ_key_len, newSViv(1), 0);
		  hv_delete(CLOSED, COORD_HV_VAL(succ_key), succ_key_len, G_DISCARD);
		}
	      }
	    }
	  }
	}
      }

    }

  }
}
