#!/usr/local/bin/perl -w
# -*- perl -*-

#
# $Id: VectorUtil.pm,v 1.7 2002/07/13 21:04:05 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 1999,2001 Slaven Rezic. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: slaven.rezic@berlin.de
# WWW:  http://bbbike.sourceforge.net/
#

package VectorUtil;

use strict;
use vars qw($VERSION $VERBOSE);
$VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);

# Diese Funktion testet, ob sich ein Vektor innerhalb eines Gitters
# befindet. Fr Vektoren, bei denen mindestens einer der Punkte innerhalb
# des Gitters ist, ist die Lsung trivial. Interessanter ist es, wenn
# beide Punkte auerhalb des Gitters sind, ein Teil des Vektors aber
# innerhalb.
# Siehe auch Grafik in ../misc/gridvectortest.fig
sub vector_in_grid {
    my($x1,$y1,$x2,$y2,$gridx1,$gridy1,$gridx2,$gridy2) = @_;

    # wenigstens ein Punkt ist innerhalb des Gitters
    return 7 if ($x1 >= $gridx1 && $x1 <= $gridx2 &&
		 $y1 >= $gridy1 && $y1 <= $gridy2);
    return 6 if ($x2 >= $gridx1 && $x2 <= $gridx2 &&
		 $y2 >= $gridy1 && $y2 <= $gridy2);

    # beide Punkte sind auerhalb des Gitters
    return 0 if $x1 < $gridx1 && $x2 < $gridx1;
    return 0 if $x1 > $gridx2 && $x2 > $gridx2;
    return 0 if $y1 < $gridy1 && $y2 < $gridy1;
    return 0 if $y1 > $gridy2 && $y2 > $gridy2;

    my $sgn;

    # Schnittpunkt-Test am rechten Rand
    my $d_x1_gridx1 = ($gridx1 - $x1);
    my $ges_strecke = sqrt(($x1-$x2)*($x1-$x2) + ($y1-$y2)*($y1-$y2));
    my $a = $d_x1_gridx1*$ges_strecke/($x2-$x1);
    my $b = sqrt($a*$a - $d_x1_gridx1*$d_x1_gridx1);
    $sgn = ($y1 < $y2 ? 1 : -1);
    $sgn *= -1 if (($x1 < $x2 && $x1 > $gridx1) ||
		   ($x2 < $x1 && $x1 < $gridx1));
    my $schnitt_y_gridx1 = $y1 + $sgn*$b;

    if ($schnitt_y_gridx1 >= $gridy1 &&
	$schnitt_y_gridx1 <= $gridy2) {
	warn "Gefunden: $gridy1 <= $schnitt_y_gridx1 <= $gridy2\n" if $VERBOSE;
	return 1;
    }


    # Schnittpunkt-Test am linken Rand
    my $d_x1_gridx2 = ($gridx2 - $x1);
    #$ges_strecke = sqrt(($x1-$x2)*($x1-$x2) + ($y1-$y2)*($y1-$y2));
    $a = $d_x1_gridx2*$ges_strecke/($x2-$x1);
    $b = sqrt($a*$a - $d_x1_gridx2*$d_x1_gridx2);
    $sgn = ($y1 < $y2 ? 1 : -1);
    $sgn *= -1 if (($x1 < $x2 && $x1 > $gridx2) ||
		   ($x2 < $x1 && $x1 < $gridx2));
    my $schnitt_y_gridx2 = $y1 + $sgn*$b;

    if ($schnitt_y_gridx2 >= $gridy1 &&
	$schnitt_y_gridx2 <= $gridy2) {
	warn "Gefunden: $gridy1 <= $schnitt_y_gridx2 <= $gridy2\n" if $VERBOSE;
	return 2;
    }

    # Schnittpunkt-Test am oberen Rand (geometrisch unten)
    my $d_y1_gridy2 = ($gridy2 - $y1);
    #$ges_strecke = sqrt(($x1-$x2)*($x1-$x2) + ($y1-$y2)*($y1-$y2));
    $a = $d_y1_gridy2*$ges_strecke/($y2-$y1);
    $b = sqrt($a*$a - $d_y1_gridy2*$d_y1_gridy2);
    $sgn = ($x1 < $x2 ? 1 : -1);
    $sgn *= -1 if (($y1 < $y2 && $y1 > $gridy2) ||
		   ($y2 < $y1 && $y1 < $gridy2));
    my $schnitt_x_gridy2 = $x1 + $sgn*$b;

    if ($schnitt_x_gridy2 >= $gridx1 &&
	$schnitt_x_gridy2 <= $gridx2) {
	warn "Gefunden: $gridx1 <= $schnitt_x_gridy2 <= $gridx2\n" if $VERBOSE;
	return 4;
    }

    # Schnittpunkt-Test am unteren Rand (geometrisch oben)
    # Der letzte Test ist nicht notwendig, weil ein Vektor das Gitter in
    # genau zwei Punkten schneiden muss, ansonsten wurde er entweder von
    # der ersten Regel erschlagen oder er geht genau durch einen Eckpunkt,
    # was fr meine Bedrfnisse uninteressant ist.
    # Der Vollstndigkeit halber:
    my $d_y1_gridy1 = ($gridy1 - $y1);
    #$ges_strecke = sqrt(($x1-$x2)*($x1-$x2) + ($y1-$y2)*($y1-$y2));
    $a = $d_y1_gridy1*$ges_strecke/($y2-$y1);
    $b = sqrt($a*$a - $d_y1_gridy1*$d_y1_gridy1);
    $sgn = ($x1 < $x2 ? 1 : -1);
    $sgn *= -1 if (($y1 < $y2 && $y1 > $gridy1) ||
		   ($y2 < $y1 && $y1 < $gridy1));
    my $schnitt_x_gridy1 = $x1 + $sgn*$b;

    if ($schnitt_x_gridy1 >= $gridx1 &&
	$schnitt_x_gridy1 <= $gridx2) {
	warn "Gefunden: $gridx1 <= $schnitt_x_gridy1 <= $gridx2\n" if $VERBOSE;
	return 3;
    }

    0;
}

# in: point ($px,$py)
#     line ($s0x,$s0y) - ($s1x,$s1y)
# out: minimum distance from point to line
sub distance_point_line {
    my($px,$py,$s0x,$s0y,$s1x,$s1y) = @_;
    my($sxd, $syd) = ($s1x-$s0x, $s1y-$s0y);
    if ($sxd+$syd==0) { # line is really a point
	return sqrt(sqr($px-$s0x)+sqr($py-$s0y));
    }
    my $tf = (($px-$s0x)*($s1x-$s0x) + ($py-$s0y)*($s1y-$s0y)) /
	     ($sxd*$sxd + $syd*$syd);
    # $nx/$ny: nearest point on line
    my $nx = $s0x+$tf*$sxd;
    my $ny = $s0y+$tf*$syd;
    if ((($nx >= $s0x && $nx <= $s1x) || ($nx >= $s1x && $nx <= $s0x))
	&&
	(($ny >= $s0y && $ny <= $s1y) || ($ny >= $s1y && $ny <= $s0y))
       ) {
	my $dx = $s0x-$px+$tf*$sxd;
	my $dy = $s0y-$py+$tf*$syd;
	sqrt($dx*$dx+$dy*$dy);
    } else {
	# nearest point is out of line ... check the endpoints of the line
	my $dist0 = sqrt(sqr($s0x-$px) + sqr($s0y-$py));
	my $dist1 = sqrt(sqr($s1x-$px) + sqr($s1y-$py));
	if ($dist0 < $dist1) {
	    $dist0;
	} else {
	    $dist1;
	}
    }
}

# REPO BEGIN
# REPO NAME sqr /home/e/eserte/src/repository 
# REPO MD5 846375a266b4452c6e0513991773b211
sub sqr { $_[0] * $_[0] }
# REPO END

1;

__END__
